﻿using System.Threading.Tasks;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using NSubstitute;
using Robware.Api.Auth.Controllers;
using Robware.Auth.API;
using Xunit;

namespace Robware.Api.Auth.Tests.Controllers {
	public class ApiControllerTests {
		[Fact]
		public async Task Validate_WithValidKey_ReturnsOk() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			apiKeyValidator.Validate("key").Returns(true);
			var controller = new ApiController(logger, apiKeyValidator);
			(await controller.Validate("key")).Should().BeOfType<OkResult>();
		}

		[Fact]
		public async Task Validate_WithInvalidKey_ReturnsUnauthorised() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			apiKeyValidator.Validate("key").Returns(false);
			var controller = new ApiController(logger, apiKeyValidator);
			(await controller.Validate("key")).Should().BeOfType<UnauthorizedResult>();
		}
	}
}