﻿using System;
using System.Threading.Tasks;
using FluentAssertions;
using NSubstitute;
using NSubstitute.ExceptionExtensions;
using Robware.Auth.API;
using Xunit;

namespace Robware.Auth.Tests.API {
	public class ApiKeyValidatorTests {
		[Fact]
		public async Task Validate_WithKeyThatExistsAndIsEnabled_ReturnsTrue() {
			var apiKey = new ApiKey {
				Name = "Test Key",
				IssueTimestamp = new DateTime(2020, 4, 19),
				Key = "3c6e0b8a9c15224a8228b9a98ca1531d",
				Enabled = true
			};

			var keys = Substitute.For<IApiKeys>();
			keys.Get("3c6e0b8a9c15224a8228b9a98ca1531d").Returns(apiKey);

			var validator = new ApiKeyValidator(keys);
			(await validator.Validate("3c6e0b8a9c15224a8228b9a98ca1531d")).Should().BeTrue();
		}

		[Fact]
		public async Task Validate_WithKeyThatExistsAndIsDisabled_ReturnsFalse() {
			var apiKey = new ApiKey {
				Name = "Test Key",
				IssueTimestamp = new DateTime(2020, 4, 19),
				Key = "3c6e0b8a9c15224a8228b9a98ca1531d",
				Enabled = false
			};

			var keys = Substitute.For<IApiKeys>();
			keys.Get("3c6e0b8a9c15224a8228b9a98ca1531d").Returns(apiKey);

			var validator = new ApiKeyValidator(keys);
			(await validator.Validate("3c6e0b8a9c15224a8228b9a98ca1531d")).Should().BeFalse();
		}

		[Fact]
		public async Task Validate_WithKeyThatDoesntExist_ReturnsFalse() {
			var keys = Substitute.For<IApiKeys>();
			keys.Get("3c6e0b8a9c15224a8228b9a98ca1531d").Throws(new ApiKeyNotFoundException(""));

			var validator = new ApiKeyValidator(keys);
			(await validator.Validate("3c6e0b8a9c15224a8228b9a98ca1531d")).Should().BeFalse();
		}
	}
}