﻿using System.Threading.Tasks;

namespace Robware.Auth.Users {
	public class Authenticator : IAuthenticator {
		private readonly IUsers _users;
		private readonly ICryptographyProvider _crypto;

		public Authenticator(IUsers users, ICryptographyProvider crypto) {
			_users = users;
			_crypto = crypto;
		}

		public async Task<(AuthenticationResult Result, User User)> Authenticate(string username, string password) {
			try {
				var user = await _users.GetByEmail(username);
				return _crypto.Encrypt(password) == user.Password ? (AuthenticationResult.Success, user) : (AuthenticationResult.IncorrectPassword, null);
			}
			catch (UserNotFoundException) {
				return (AuthenticationResult.NotFound, null);
			}
		}
	}

	public enum AuthenticationResult {
		Unknown,
		Success,
		NotFound,
		IncorrectPassword
	}
}